@@ -39,6 +39,8 @@
 #include "vulkan/runtime/vk_shader_module.h"
 #include "vulkan/vulkan.h"
 
+#include "aco_shader_info.h"
+
 #define RADV_VERT_ATTRIB_MAX MAX2(VERT_ATTRIB_MAX, VERT_ATTRIB_GENERIC0 + MAX_VERTEX_ATTRIBS)
 
 struct radv_physical_device;
@@ -46,14 +48,9 @@ struct radv_device;
 struct radv_pipeline;
 struct radv_pipeline_cache;
 struct radv_pipeline_key;
+struct radv_shader_args;
 struct radv_vs_input_state;
-
-enum radv_vs_input_alpha_adjust {
-   ALPHA_ADJUST_NONE = 0,
-   ALPHA_ADJUST_SNORM = 1,
-   ALPHA_ADJUST_SSCALED = 2,
-   ALPHA_ADJUST_SINT = 3,
-};
+struct radv_shader_args;
 
 struct radv_pipeline_key {
    uint32_t has_multiview_view_index : 1;
@@ -62,6 +59,11 @@ struct radv_pipeline_key {
    uint32_t use_ngg : 1;
    uint32_t adjust_frag_coord_z : 1;
    uint32_t disable_aniso_single_level : 1;
+   uint32_t disable_sinking_load_input_fs : 1;
+   uint32_t image_2d_view_of_3d : 1;
+   uint32_t primitives_generated_query : 1;
+   uint32_t dynamic_patch_control_points : 1;
+   uint32_t dynamic_rasterization_samples : 1;
 
    struct {
       uint32_t instance_rate_inputs;
@@ -71,10 +73,8 @@ struct radv_pipeline_key {
       uint32_t vertex_attribute_offsets[MAX_VERTEX_ATTRIBS];
       uint32_t vertex_attribute_strides[MAX_VERTEX_ATTRIBS];
       uint8_t vertex_binding_align[MAX_VBS];
-      enum radv_vs_input_alpha_adjust vertex_alpha_adjust[MAX_VERTEX_ATTRIBS];
-      uint32_t vertex_post_shuffle;
       uint32_t provoking_vtx_last : 1;
-      uint32_t dynamic_input_state : 1;
+      uint32_t has_prolog : 1;
       uint8_t topology;
    } vs;
 
@@ -86,12 +86,19 @@ struct radv_pipeline_key {
       uint32_t col_format;
       uint32_t is_int8;
       uint32_t is_int10;
+      uint32_t cb_target_mask;
       uint8_t log2_ps_iter_samples;
       uint8_t num_samples;
+      bool mrt0_is_dual_src;
 
       bool lower_discard_to_demote;
-      bool enable_mrt_output_nan_fixup;
-      uint8_t force_vrs;
+      uint8_t enable_mrt_output_nan_fixup;
+      bool force_vrs_enabled;
+
+      /* Used to export alpha through MRTZ for alpha-to-coverage (GFX11+). */
+      bool alpha_to_coverage_via_mrtz;
+
+      bool has_epilog;
    } ps;
 
    struct {
@@ -103,36 +110,24 @@ struct radv_pipeline_key {
    } cs;
 };
 
-enum radv_compiler_debug_level {
-   RADV_COMPILER_DEBUG_LEVEL_PERFWARN,
-   RADV_COMPILER_DEBUG_LEVEL_ERROR,
-};
-
 struct radv_nir_compiler_options {
-   struct radv_pipeline_layout *layout;
    struct radv_pipeline_key key;
-   bool explicit_scratch_args;
    bool robust_buffer_access;
-   bool adjust_frag_coord_z;
    bool dump_shader;
    bool dump_preoptir;
    bool record_ir;
    bool record_stats;
    bool check_ir;
    bool has_ls_vgpr_init_bug;
-   bool has_image_load_dcc_bug;
-   bool enable_mrt_output_nan_fixup;
+   uint8_t enable_mrt_output_nan_fixup;
    bool wgp_mode;
-   bool remap_spi_ps_input;
-   bool disable_aniso_single_level;
    enum radeon_family family;
-   enum chip_class chip_class;
-   const struct radeon_info *info;
+   enum amd_gfx_level gfx_level;
    uint32_t address32_hi;
-   uint8_t force_vrs_rates;
+   bool has_3d_cube_border_color_mipmap;
 
    struct {
-      void (*func)(void *private_data, enum radv_compiler_debug_level level, const char *message);
+      void (*func)(void *private_data, enum aco_compiler_debug_level level, const char *message);
       void *private_data;
    } debug;
 };
@@ -144,23 +139,33 @@ enum radv_ud_index {
    AC_UD_INDIRECT_DESCRIPTOR_SETS = 3,
    AC_UD_VIEW_INDEX = 4,
    AC_UD_STREAMOUT_BUFFERS = 5,
-   AC_UD_NGG_GS_STATE = 6,
+   AC_UD_NGG_QUERY_STATE = 6,
    AC_UD_NGG_CULLING_SETTINGS = 7,
    AC_UD_NGG_VIEWPORT = 8,
-   AC_UD_SHADER_START = 9,
+   AC_UD_FORCE_VRS_RATES = 9,
+   AC_UD_TASK_RING_ENTRY = 10,
+   AC_UD_SHADER_START = 11,
    AC_UD_VS_VERTEX_BUFFERS = AC_UD_SHADER_START,
    AC_UD_VS_BASE_VERTEX_START_INSTANCE,
    AC_UD_VS_PROLOG_INPUTS,
    AC_UD_VS_MAX_UD,
+   AC_UD_PS_EPILOG_PC,
+   AC_UD_PS_NUM_SAMPLES,
    AC_UD_PS_MAX_UD,
    AC_UD_CS_GRID_SIZE = AC_UD_SHADER_START,
    AC_UD_CS_SBT_DESCRIPTORS,
-   AC_UD_CS_RAY_LAUNCH_SIZE,
+   AC_UD_CS_RAY_LAUNCH_SIZE_ADDR,
+   AC_UD_CS_RAY_DYNAMIC_CALLABLE_STACK_BASE,
+   AC_UD_CS_TASK_RING_OFFSETS,
+   AC_UD_CS_TASK_DRAW_ID,
+   AC_UD_CS_TASK_IB,
    AC_UD_CS_MAX_UD,
    AC_UD_GS_MAX_UD,
+   AC_UD_TCS_OFFCHIP_LAYOUT = AC_UD_VS_MAX_UD,
    AC_UD_TCS_MAX_UD,
+   AC_UD_TES_NUM_PATCHES = AC_UD_SHADER_START,
    AC_UD_TES_MAX_UD,
-   AC_UD_MAX_UD = AC_UD_TCS_MAX_UD,
+   AC_UD_MAX_UD = AC_UD_CS_MAX_UD,
 };
 
 struct radv_stream_output {
@@ -203,15 +208,10 @@ struct radv_vs_output_info {
    bool writes_primitive_shading_rate;
    bool writes_primitive_shading_rate_per_primitive;
    bool export_prim_id;
-   bool export_prim_id_per_primitive;
    bool export_clip_dists;
    unsigned pos_exports;
 };
 
-struct radv_es_output_info {
-   uint32_t esgs_itemsize;
-};
-
 struct gfx9_gs_info {
    uint32_t vgt_gs_onchip_cntl;
    uint32_t vgt_gs_max_prims_per_subgroup;
@@ -227,17 +227,15 @@ struct gfx10_ngg_info {
    uint32_t prim_amp_factor;
    uint32_t vgt_esgs_ring_itemsize;
    uint32_t esgs_ring_size;
+   uint32_t scratch_lds_base;
    bool max_vert_out_per_gs_instance;
-   bool enable_vertex_grouping;
 };
 
 struct radv_shader_info {
+   uint64_t inline_push_constant_mask;
+   bool can_inline_all_push_constants;
    bool loads_push_constants;
    bool loads_dynamic_offsets;
-   uint16_t min_push_constant_used;
-   uint16_t max_push_constant_used;
-   bool has_only_32bit_push_constants;
-   bool has_indirect_push_constants;
    uint32_t desc_set_used_mask;
    bool uses_view_index;
    bool uses_invocation_id;
@@ -249,16 +247,19 @@ struct radv_shader_info {
    bool is_ngg_passthrough;
    bool has_ngg_culling;
    bool has_ngg_early_prim_export;
+   bool has_ngg_prim_query;
+   bool has_ngg_xfb_query;
    uint32_t num_lds_blocks_when_not_culling;
    uint32_t num_tess_patches;
+   uint32_t esgs_itemsize; /* Only for VS or TES as ES */
+   struct radv_vs_output_info outinfo;
    unsigned workgroup_size;
+   bool force_vrs_per_vertex;
    struct {
       uint8_t input_usage_mask[RADV_VERT_ATTRIB_MAX];
       uint8_t output_usage_mask[VARYING_SLOT_VAR31 + 1];
       bool needs_draw_id;
       bool needs_instance_id;
-      struct radv_vs_output_info outinfo;
-      struct radv_es_output_info es_info;
       bool as_es;
       bool as_ls;
       bool tcs_in_out_eq;
@@ -267,6 +268,7 @@ struct radv_shader_info {
       bool needs_base_instance;
       bool use_per_attribute_vb_descs;
       uint32_t vb_desc_usage_mask;
+      uint32_t input_slot_usage_mask;
       bool has_prolog;
       bool dynamic_inputs;
    } vs;
@@ -279,15 +281,15 @@ struct radv_shader_info {
       unsigned max_gsvs_emit_size;
       unsigned vertices_in;
       unsigned vertices_out;
+      unsigned input_prim;
       unsigned output_prim;
       unsigned invocations;
       unsigned es_type; /* GFX9: VS or TES */
       uint8_t num_linked_inputs;
+      bool has_ngg_pipeline_stat_query;
    } gs;
    struct {
       uint8_t output_usage_mask[VARYING_SLOT_VAR31 + 1];
-      struct radv_vs_output_info outinfo;
-      struct radv_es_output_info es_info;
       bool as_es;
       enum tess_primitive_mode _primitive_mode;
       enum gl_tess_spacing spacing;
@@ -334,7 +336,9 @@ struct radv_shader_info {
       uint8_t reads_sample_pos_mask;
       uint8_t depth_layout;
       bool allow_flat_shading;
+      bool has_epilog;
       unsigned spi_ps_input;
+      unsigned colors_written;
    } ps;
    struct {
       bool uses_grid_size;
@@ -347,6 +351,7 @@ struct radv_shader_info {
 
       bool uses_sbt;
       bool uses_ray_launch_size;
+      bool uses_dynamic_rt_callable_stack;
    } cs;
    struct {
       uint64_t tes_inputs_read;
@@ -359,8 +364,9 @@ struct radv_shader_info {
       bool tes_reads_tess_factors : 1;
    } tcs;
    struct {
-      struct radv_vs_output_info outinfo;
       enum shader_prim output_prim;
+      bool needs_ms_scratch_ring;
+      bool has_task; /* If mesh shader is used together with a task shader. */
    } ms;
 
    struct radv_streamout_info so;
@@ -371,17 +377,17 @@ struct radv_shader_info {
 
 struct radv_vs_input_state {
    uint32_t attribute_mask;
-   uint32_t misaligned_mask;
-   uint32_t possibly_misaligned_mask;
 
    uint32_t instance_rate_inputs;
    uint32_t nontrivial_divisors;
+   uint32_t zero_divisors;
    uint32_t post_shuffle;
    /* Having two separate fields instead of a single uint64_t makes it easier to remove attributes
     * using bitwise arithmetic.
     */
    uint32_t alpha_adjust_lo;
    uint32_t alpha_adjust_hi;
+   uint32_t nontrivial_formats;
 
    uint8_t bindings[MAX_VERTEX_ATTRIBS];
    uint32_t divisors[MAX_VERTEX_ATTRIBS];
@@ -389,6 +395,8 @@ struct radv_vs_input_state {
    uint8_t formats[MAX_VERTEX_ATTRIBS];
    uint8_t format_align_req_minus_1[MAX_VERTEX_ATTRIBS];
    uint8_t format_sizes[MAX_VERTEX_ATTRIBS];
+
+   bool bindings_match_attrib;
 };
 
 struct radv_vs_prolog_key {
@@ -401,6 +409,18 @@ struct radv_vs_prolog_key {
    gl_shader_stage next_stage;
 };
 
+struct radv_ps_epilog_key {
+   uint32_t spi_shader_col_format;
+
+   /* Bitmasks, each bit represents one of the 8 MRTs. */
+   uint8_t color_is_int8;
+   uint8_t color_is_int10;
+   uint8_t enable_mrt_output_nan_fixup;
+
+   bool mrt0_is_dual_src;
+   bool wave32;
+};
+
 enum radv_shader_binary_type { RADV_BINARY_TYPE_LEGACY, RADV_BINARY_TYPE_RTLD };
 
 struct radv_shader_binary {
@@ -435,7 +455,7 @@ struct radv_shader_binary_rtld {
    uint8_t data[0];
 };
 
-struct radv_prolog_binary {
+struct radv_shader_part_binary {
    uint8_t num_sgprs;
    uint8_t num_vgprs;
    uint8_t num_preserved_sgprs;
@@ -469,6 +489,7 @@ union radv_shader_arena_block {
 struct radv_shader {
    uint32_t ref_count;
 
+   struct radeon_winsys_bo *bo; /* Not NULL if imported from a lib */
    uint64_t va;
 
    struct ac_shader_config config;
@@ -477,7 +498,7 @@ struct radv_shader {
    uint32_t exec_size;
    struct radv_shader_info info;
 
-   /* debug only */
+   /* debug only */
    char *spirv;
    uint32_t spirv_size;
    char *nir_string;
@@ -486,39 +507,69 @@ struct radv_shader {
    uint32_t *statistics;
 };
 
-struct radv_shader_prolog {
+struct radv_trap_handler_shader {
    struct radeon_winsys_bo *bo;
    union radv_shader_arena_block *alloc;
+};
+
+struct radv_shader_part {
+   uint32_t ref_count;
+
+   uint64_t va;
+
+   struct radeon_winsys_bo *bo;
+   union radv_shader_arena_block *alloc;
+   uint32_t code_size;
    uint32_t rsrc1;
    uint8_t num_preserved_sgprs;
    bool nontrivial_divisors;
 
+   struct radv_shader_part_binary *binary;
+
    /* debug only */
    char *disasm_string;
 };
 
-void radv_optimize_nir(const struct radv_device *device, struct nir_shader *shader,
-                       bool optimize_conservatively, bool allow_copies);
+struct radv_pipeline_layout;
+
+void radv_optimize_nir(struct nir_shader *shader, bool optimize_conservatively, bool allow_copies);
 void radv_optimize_nir_algebraic(nir_shader *shader, bool opt_offsets);
 bool radv_nir_lower_ycbcr_textures(nir_shader *shader, const struct radv_pipeline_layout *layout);
 
-nir_shader *radv_shader_compile_to_nir(struct radv_device *device, struct vk_shader_module *module,
-                                       const char *entrypoint_name, gl_shader_stage stage,
-                                       const VkSpecializationInfo *spec_info,
-                                       const struct radv_pipeline_layout *layout,
-                                       const struct radv_pipeline_key *key);
+bool radv_nir_lower_ray_queries(nir_shader *shader, struct radv_device *device);
+
+void radv_nir_apply_pipeline_layout(nir_shader *shader, struct radv_device *device,
+                                    const struct radv_pipeline_layout *layout,
+                                    const struct radv_shader_info *info,
+                                    const struct radv_shader_args *args);
+
+struct radv_pipeline_stage;
+
+nir_shader *radv_shader_spirv_to_nir(struct radv_device *device,
+                                     const struct radv_pipeline_stage *stage,
+                                     const struct radv_pipeline_key *key);
+
+void radv_nir_lower_abi(nir_shader *shader, enum amd_gfx_level gfx_level,
+                        const struct radv_shader_info *info, const struct radv_shader_args *args,
+                        const struct radv_pipeline_key *pl_key, bool use_llvm,
+                        uint32_t address32_hi);
 
 void radv_init_shader_arenas(struct radv_device *device);
 void radv_destroy_shader_arenas(struct radv_device *device);
 
+struct radv_pipeline_shader_stack_size;
+
 VkResult radv_create_shaders(struct radv_pipeline *pipeline,
                              struct radv_pipeline_layout *pipeline_layout,
                              struct radv_device *device, struct radv_pipeline_cache *cache,
                              const struct radv_pipeline_key *key,
-                             const VkPipelineShaderStageCreateInfo **pStages,
+                             const VkPipelineShaderStageCreateInfo *pStages,
+                             uint32_t stageCount,
                              const VkPipelineCreateFlags flags, const uint8_t *custom_hash,
-                             VkPipelineCreationFeedback *pipeline_feedback,
-                             VkPipelineCreationFeedback **stage_feedbacks);
+                             const VkPipelineCreationFeedbackCreateInfo *creation_feedback,
+                             struct radv_pipeline_shader_stack_size **stack_sizes,
+                             uint32_t *num_stack_sizes,
+                             gl_shader_stage *last_vgt_api_stage);
 
 struct radv_shader_args;
 
@@ -526,33 +577,41 @@ struct radv_shader *radv_shader_create(struct radv_device *device,
                                        const struct radv_shader_binary *binary,
                                        bool keep_shader_info, bool from_cache,
                                        const struct radv_shader_args *args);
-struct radv_shader *radv_shader_compile(
-   struct radv_device *device, struct vk_shader_module *module, struct nir_shader *const *shaders,
-   int shader_count, struct radv_pipeline_layout *layout, const struct radv_pipeline_key *key,
-   struct radv_shader_info *info, bool keep_shader_info, bool keep_statistic_info,
+struct radv_shader *radv_shader_nir_to_asm(
+   struct radv_device *device, struct radv_pipeline_stage *stage, struct nir_shader *const *shaders,
+   int shader_count, const struct radv_pipeline_key *key, bool keep_shader_info, bool keep_statistic_info,
    struct radv_shader_binary **binary_out);
 
 bool radv_shader_binary_upload(struct radv_device *device, const struct radv_shader_binary *binary,
                                struct radv_shader *shader, void *dest_ptr);
 
+void radv_shader_part_binary_upload(const struct radv_shader_part_binary *binary, void *dest_ptr);
+
 union radv_shader_arena_block *radv_alloc_shader_memory(struct radv_device *device, uint32_t size,
                                                         void *ptr);
 void radv_free_shader_memory(struct radv_device *device, union radv_shader_arena_block *alloc);
 
 struct radv_shader *
 radv_create_gs_copy_shader(struct radv_device *device, struct nir_shader *nir,
-                           struct radv_shader_info *info, struct radv_shader_binary **binary_out,
-                           bool multiview, bool keep_shader_info, bool keep_statistic_info,
+                           const struct radv_shader_info *info, const struct radv_shader_args *args,
+                           struct radv_shader_binary **binary_out,
+                           bool keep_shader_info, bool keep_statistic_info,
                            bool disable_optimizations);
 
-struct radv_shader *radv_create_trap_handler_shader(struct radv_device *device);
+struct radv_trap_handler_shader *radv_create_trap_handler_shader(struct radv_device *device);
+uint64_t radv_trap_handler_shader_get_va(const struct radv_trap_handler_shader *trap);
+void radv_trap_handler_shader_destroy(struct radv_device *device,
+                                      struct radv_trap_handler_shader *trap);
 
-struct radv_shader_prolog *radv_create_vs_prolog(struct radv_device *device,
-                                                 const struct radv_vs_prolog_key *key);
+struct radv_shader_part *radv_create_vs_prolog(struct radv_device *device,
+                                               const struct radv_vs_prolog_key *key);
+
+struct radv_shader_part *radv_create_ps_epilog(struct radv_device *device,
+                                               const struct radv_ps_epilog_key *key);
 
 void radv_shader_destroy(struct radv_device *device, struct radv_shader *shader);
 
-void radv_prolog_destroy(struct radv_device *device, struct radv_shader_prolog *prolog);
+void radv_shader_part_destroy(struct radv_device *device, struct radv_shader_part *shader_part);
 
 uint64_t radv_shader_get_va(const struct radv_shader *shader);
 struct radv_shader *radv_find_shader(struct radv_device *device, uint64_t pc);
@@ -562,19 +621,50 @@ unsigned radv_get_max_waves(const struct radv_device *device, struct radv_shader
 
 const char *radv_get_shader_name(const struct radv_shader_info *info, gl_shader_stage stage);
 
-unsigned radv_compute_spi_ps_input(const struct radv_device *device,
+unsigned radv_compute_spi_ps_input(const struct radv_pipeline_key *pipeline_key,
                                    const struct radv_shader_info *info);
 
-bool radv_can_dump_shader(struct radv_device *device, struct vk_shader_module *module,
-                          bool meta_shader);
+bool radv_can_dump_shader(struct radv_device *device, nir_shader *nir, bool meta_shader);
 
-bool radv_can_dump_shader_stats(struct radv_device *device, struct vk_shader_module *module);
+bool radv_can_dump_shader_stats(struct radv_device *device, nir_shader *nir);
 
 VkResult radv_dump_shader_stats(struct radv_device *device, struct radv_pipeline *pipeline,
                                 gl_shader_stage stage, FILE *output);
 
+static inline struct radv_shader *
+radv_shader_ref(struct radv_shader *shader)
+{
+   assert(shader && shader->ref_count >= 1);
+   p_atomic_inc(&shader->ref_count);
+   return shader;
+}
+
+static inline void
+radv_shader_unref(struct radv_device *device, struct radv_shader *shader)
+{
+   assert(shader && shader->ref_count >= 1);
+   if (p_atomic_dec_zero(&shader->ref_count))
+      radv_shader_destroy(device, shader);
+}
+
+static inline struct radv_shader_part *
+radv_shader_part_ref(struct radv_shader_part *shader_part)
+{
+   assert(shader_part && shader_part->ref_count >= 1);
+   p_atomic_inc(&shader_part->ref_count);
+   return shader_part;
+}
+
+static inline void
+radv_shader_part_unref(struct radv_device *device, struct radv_shader_part *shader_part)
+{
+   assert(shader_part && shader_part->ref_count >= 1);
+   if (p_atomic_dec_zero(&shader_part->ref_count))
+      radv_shader_part_destroy(device, shader_part);
+}
+
 static inline unsigned
-calculate_tess_lds_size(enum chip_class chip_class, unsigned tcs_num_input_vertices,
+calculate_tess_lds_size(enum amd_gfx_level gfx_level, unsigned tcs_num_input_vertices,
                         unsigned tcs_num_output_vertices, unsigned tcs_num_inputs,
                         unsigned tcs_num_patches, unsigned tcs_num_outputs,
                         unsigned tcs_num_patch_outputs)
@@ -591,7 +681,7 @@ calculate_tess_lds_size(enum chip_class chip_class, unsigned tcs_num_input_verti
 
    unsigned lds_size = output_patch0_offset + output_patch_size * tcs_num_patches;
 
-   if (chip_class >= GFX7) {
+   if (gfx_level >= GFX7) {
       assert(lds_size <= 65536);
       lds_size = align(lds_size, 512) / 512;
    } else {
@@ -606,7 +696,7 @@ static inline unsigned
 get_tcs_num_patches(unsigned tcs_num_input_vertices, unsigned tcs_num_output_vertices,
                     unsigned tcs_num_inputs, unsigned tcs_num_outputs,
                     unsigned tcs_num_patch_outputs, unsigned tess_offchip_block_dw_size,
-                    enum chip_class chip_class, enum radeon_family family)
+                    enum amd_gfx_level gfx_level, enum radeon_family family)
 {
    uint32_t input_vertex_size = tcs_num_inputs * 16;
    uint32_t input_patch_size = tcs_num_input_vertices * input_vertex_size;
@@ -629,7 +719,7 @@ get_tcs_num_patches(unsigned tcs_num_input_vertices, unsigned tcs_num_output_ver
     *
     * Test: dEQP-VK.tessellation.shader_input_output.barrier
     */
-   if (chip_class >= GFX7 && family != CHIP_STONEY)
+   if (gfx_level >= GFX7 && family != CHIP_STONEY)
       hardware_lds_size = 65536;
 
    if (input_patch_size + output_patch_size)
@@ -643,7 +733,7 @@ get_tcs_num_patches(unsigned tcs_num_input_vertices, unsigned tcs_num_output_ver
    num_patches = MIN2(num_patches, 40);
 
    /* GFX6 bug workaround - limit LS-HS threadgroups to only one wave. */
-   if (chip_class == GFX6) {
+   if (gfx_level == GFX6) {
       unsigned one_wave = 64 / MAX2(tcs_num_input_vertices, tcs_num_output_vertices);
       num_patches = MIN2(num_patches, one_wave);
    }
@@ -652,17 +742,22 @@ get_tcs_num_patches(unsigned tcs_num_input_vertices, unsigned tcs_num_output_ver
 
 void radv_lower_io(struct radv_device *device, nir_shader *nir);
 
-bool radv_lower_io_to_mem(struct radv_device *device, struct nir_shader *nir,
-                          const struct radv_shader_info *info, const struct radv_pipeline_key *pl_key);
+bool radv_lower_io_to_mem(struct radv_device *device, struct radv_pipeline_stage *stage);
 
-void radv_lower_ngg(struct radv_device *device, struct nir_shader *nir,
-                    const struct radv_shader_info *info,
+bool radv_lower_view_index(nir_shader *nir, bool per_primitive);
+
+void radv_lower_ngg(struct radv_device *device, struct radv_pipeline_stage *ngg_stage,
                     const struct radv_pipeline_key *pl_key);
 
-bool radv_consider_culling(struct radv_device *device, struct nir_shader *nir,
+bool radv_consider_culling(const struct radv_physical_device *pdevice, struct nir_shader *nir,
                            uint64_t ps_inputs_read, unsigned num_vertices_per_primitive,
                            const struct radv_shader_info *info);
 
 void radv_get_nir_options(struct radv_physical_device *device);
 
+bool radv_force_primitive_shading_rate(nir_shader *nir, struct radv_device *device);
+
+bool radv_lower_fs_intrinsics(nir_shader *nir, const struct radv_pipeline_stage *fs_stage,
+                              const struct radv_pipeline_key *key);
+
 #endif
