@@ -50,6 +50,7 @@
 #include "vk_command_buffer.h"
 #include "vk_command_pool.h"
 #include "vk_queue.h"
+#include "vk_pipeline.h"
 
 #include <xf86drm.h>
 
@@ -103,7 +104,7 @@
 #endif
 
 #define perf_debug(...) do {                       \
-   if (unlikely(V3D_DEBUG & V3D_DEBUG_PERF))       \
+   if (V3D_DBG(PERF))                            \
       fprintf(stderr, __VA_ARGS__);                \
 } while (0)
 
@@ -165,6 +166,8 @@ struct v3dv_physical_device {
    const struct v3d_compiler *compiler;
    uint32_t next_program_id;
 
+   uint64_t heap_used;
+
    /* This array holds all our 'struct v3dv_bo' allocations. We use this
     * so we can add a refcount to our BOs and check if a particular BO
     * was already allocated in this device using its GEM handle. This is
@@ -223,9 +226,6 @@ bool v3dv_meta_can_use_tlb(struct v3dv_image *image,
 struct v3dv_instance {
    struct vk_instance vk;
 
-   int physicalDeviceCount;
-   struct v3dv_physical_device physicalDevice;
-
    bool pipeline_cache_enabled;
    bool default_pipeline_cache_enabled;
 };
@@ -313,7 +313,6 @@ struct v3dv_meta_texel_buffer_copy_pipeline {
 };
 
 struct v3dv_pipeline_key {
-   bool robust_buffer_access;
    uint8_t topology;
    uint8_t logicop_func;
    bool msaa;
@@ -455,6 +454,15 @@ struct v3dv_pipeline_cache {
    bool externally_synchronized;
 };
 
+/* This is used to implement a list of free events in the BO we use
+ * hold event states. The index here is used to calculate the offset
+ * within that BO.
+ */
+struct v3dv_event_desc {
+   struct list_head link;
+   uint32_t index;
+};
+
 struct v3dv_device {
    struct vk_device vk;
 
@@ -510,6 +518,44 @@ struct v3dv_device {
    uint32_t bo_size;
    uint32_t bo_count;
 
+   /* Event handling resources.
+    *
+    * Our implementation of events uses a BO to store event state (signaled vs
+    * reset) and dispatches compute shaders to handle GPU event functions
+    * (signal, reset, wait). This struct holds all the resources required
+    * by the implementation.
+    */
+   struct {
+      mtx_t lock;
+
+      /* BO for the event states: signaled (1) or reset (0) */
+      struct v3dv_bo *bo;
+
+      /* Events can be created and destroyed. Since we have a dedicated BO for
+       * all events we use, we need to keep track of the free slots within that
+       * BO. For that we use a free list where we link together available event
+       * slots in the form of "descriptors" that include an index (which is
+       * basically an offset into the BO that is available).
+       */
+      uint32_t desc_count;
+      struct v3dv_event_desc *desc;
+      struct list_head free_list;
+
+      /* Vulkan resources to access the event BO from shaders. We have a
+       * pipeline that sets the state of an event and another that waits on
+       * a single event. Both pipelines require access to the event state BO,
+       * for which we need to allocate a single descripot set.
+       */
+      VkBuffer buffer;
+      VkDeviceMemory mem;
+      VkDescriptorSetLayout descriptor_set_layout;
+      VkPipelineLayout pipeline_layout;
+      VkDescriptorPool descriptor_pool;
+      VkDescriptorSet descriptor_set;
+      VkPipeline set_event_pipeline;
+      VkPipeline wait_event_pipeline;
+   } events;
+
    struct v3dv_pipeline_cache default_pipeline_cache;
 
    /* GL_SHADER_STATE_RECORD needs to speficy default attribute values. The
@@ -519,7 +565,6 @@ struct v3dv_device {
     * attributes will create their own BO.
     */
    struct v3dv_bo *default_attribute_float;
-   VkPhysicalDeviceFeatures features;
 
    void *device_address_mem_ctx;
    struct util_dynarray device_address_bo_list; /* Array of struct v3dv_bo * */
@@ -607,6 +652,12 @@ struct v3dv_image {
 #endif
 };
 
+VkResult
+v3dv_image_init(struct v3dv_device *device,
+                const VkImageCreateInfo *pCreateInfo,
+                const VkAllocationCallbacks *pAllocator,
+                struct v3dv_image *image);
+
 VkImageViewType v3dv_image_type_to_view_type(VkImageType type);
 
 /* Pre-generating packets needs to consider changes in packet sizes across hw
@@ -964,8 +1015,6 @@ enum v3dv_job_type {
    V3DV_JOB_TYPE_CPU_RESET_QUERIES,
    V3DV_JOB_TYPE_CPU_END_QUERY,
    V3DV_JOB_TYPE_CPU_COPY_QUERY_RESULTS,
-   V3DV_JOB_TYPE_CPU_SET_EVENT,
-   V3DV_JOB_TYPE_CPU_WAIT_EVENTS,
    V3DV_JOB_TYPE_CPU_COPY_BUFFER_TO_IMAGE,
    V3DV_JOB_TYPE_CPU_CSD_INDIRECT,
    V3DV_JOB_TYPE_CPU_TIMESTAMP_QUERY,
@@ -1005,17 +1054,6 @@ struct v3dv_submit_sync_info {
    struct vk_sync_signal *signals;
 };
 
-struct v3dv_event_set_cpu_job_info {
-   struct v3dv_event *event;
-   int state;
-};
-
-struct v3dv_event_wait_cpu_job_info {
-   /* List of events to wait on */
-   uint32_t event_count;
-   struct v3dv_event **events;
-};
-
 struct v3dv_copy_buffer_to_image_cpu_job_info {
    struct v3dv_image *image;
    struct v3dv_buffer *buffer;
@@ -1080,6 +1118,37 @@ struct v3dv_job {
     */
    bool uses_buffer_device_address;
 
+   /* True if we have not identified anything that would be incompatible
+    * with double-buffer (like MSAA) or that would make double-buffer mode
+    * not efficient (like tile loads or not having any stores).
+    */
+   bool can_use_double_buffer;
+
+   /* This structure keeps track of various scores to inform a heuristic
+    * for double-buffer mode.
+    */
+   struct {
+      /* Cost of geometry shading */
+      uint32_t geom;
+      /* Cost of shader rendering */
+      uint32_t render;
+   } double_buffer_score;
+
+   /* We only need to allocate tile state for all layers if the binner
+    * writes primitives to layers other than the first. This can only be
+    * done using layered rendering (writing gl_Layer from a geometry shader),
+    * so for other cases of multilayered framebuffers (typically with
+    * meta copy/clear operations) that won't use layered rendering, we only
+    * need one layer worth of of tile state for the binner.
+    */
+   bool allocate_tile_state_for_all_layers;
+
+   /* A pointer to the location of the TILE_BINNING_MODE_CFG packet so we can
+    * rewrite it to enable double-buffer mode by the time we have enough info
+    * about the job to make that decision.
+    */
+   struct v3dv_cl_out *bcl_tile_binning_mode_ptr;
+
    enum v3dv_job_type type;
 
    struct v3dv_device *device;
@@ -1151,8 +1220,6 @@ struct v3dv_job {
       struct v3dv_reset_query_cpu_job_info          query_reset;
       struct v3dv_end_query_cpu_job_info            query_end;
       struct v3dv_copy_query_results_cpu_job_info   query_copy_results;
-      struct v3dv_event_set_cpu_job_info            event_set;
-      struct v3dv_event_wait_cpu_job_info           event_wait;
       struct v3dv_copy_buffer_to_image_cpu_job_info copy_buffer_to_image;
       struct v3dv_csd_indirect_cpu_job_info         csd_indirect;
       struct v3dv_timestamp_query_cpu_job_info      query_timestamp;
@@ -1188,6 +1255,7 @@ void v3dv_job_start_frame(struct v3dv_job *job,
                           uint32_t height,
                           uint32_t layers,
                           bool allocate_tile_state_for_all_layers,
+                          bool allocate_tile_state_now,
                           uint32_t render_target_count,
                           uint8_t max_internal_bpp,
                           bool msaa);
@@ -1211,7 +1279,10 @@ v3dv_cmd_buffer_ensure_array_state(struct v3dv_cmd_buffer *cmd_buffer,
                                    void **ptr);
 
 void v3dv_cmd_buffer_emit_pre_draw(struct v3dv_cmd_buffer *cmd_buffer,
-                                   bool indexed, bool indirect);
+                                   bool indexed, bool indirect,
+                                   uint32_t vertex_count);
+
+bool v3dv_job_allocate_tile_state(struct v3dv_job *job);
 
 /* FIXME: only used on v3dv_cmd_buffer and v3dvx_cmd_buffer, perhaps move to a
  * cmd_buffer specific header?
@@ -1263,8 +1334,8 @@ struct v3dv_barrier_state {
    /* For graphics barriers, access masks involved. Used to decide if we need
     * to execute a binning or render barrier.
     */
-   VkAccessFlags bcl_buffer_access;
-   VkAccessFlags bcl_image_access;
+   VkAccessFlags2 bcl_buffer_access;
+   VkAccessFlags2 bcl_image_access;
 };
 
 struct v3dv_cmd_buffer_state {
@@ -1394,6 +1465,11 @@ struct v3dv_cmd_buffer_state {
    } query;
 };
 
+void
+v3dv_cmd_buffer_state_get_viewport_z_xform(struct v3dv_cmd_buffer_state *state,
+                                           uint32_t vp_idx,
+                                           float *translate_z, float *scale_z);
+
 /* The following struct represents the info from a descriptor that we store on
  * the host memory. They are mostly links to other existing vulkan objects,
  * like the image_view in order to access to swizzle info, or the buffer used
@@ -1483,6 +1559,8 @@ struct v3dv_cmd_buffer_private_obj {
    v3dv_cmd_buffer_private_obj_destroy_cb destroy_cb;
 };
 
+extern const struct vk_command_buffer_ops v3dv_cmd_buffer_ops;
+
 struct v3dv_cmd_buffer {
    struct vk_command_buffer vk;
 
@@ -1583,11 +1661,36 @@ void v3dv_cmd_buffer_add_private_obj(struct v3dv_cmd_buffer *cmd_buffer,
 void v3dv_cmd_buffer_merge_barrier_state(struct v3dv_barrier_state *dst,
                                          struct v3dv_barrier_state *src);
 
+bool v3dv_cmd_buffer_check_needs_load(const struct v3dv_cmd_buffer_state *state,
+                                      VkImageAspectFlags aspect,
+                                      uint32_t first_subpass_idx,
+                                      VkAttachmentLoadOp load_op,
+                                      uint32_t last_subpass_idx,
+                                      VkAttachmentStoreOp store_op);
+
+bool v3dv_cmd_buffer_check_needs_store(const struct v3dv_cmd_buffer_state *state,
+                                       VkImageAspectFlags aspect,
+                                       uint32_t last_subpass_idx,
+                                       VkAttachmentStoreOp store_op);
+
+void v3dv_cmd_buffer_emit_pipeline_barrier(struct v3dv_cmd_buffer *cmd_buffer,
+                                           const VkDependencyInfoKHR *info);
+
 struct v3dv_event {
    struct vk_object_base base;
-   int state;
+
+   /* Each event gets a different index, which we use to compute the offset
+    * in the BO we use to track their state (signaled vs reset).
+    */
+   uint32_t index;
 };
 
+VkResult
+v3dv_event_allocate_resources(struct v3dv_device *device);
+
+void
+v3dv_event_free_resources(struct v3dv_device *device);
+
 struct v3dv_shader_variant {
    enum broadcom_shader_stage stage;
 
@@ -1609,9 +1712,11 @@ struct v3dv_shader_variant {
     */
    uint32_t assembly_offset;
 
-   /* Note: it is really likely that qpu_insts would be NULL, as it will be
-    * used only temporarily, to upload it to the shared bo, as we compile the
-    * different stages individually.
+   /* Note: don't assume qpu_insts to be always NULL or not-NULL. In general
+    * we will try to free it as soon as we upload it to the shared bo while we
+    * compile the different stages. But we can decide to keep it around based
+    * on some pipeline creation flags, like
+    * VK_PIPELINE_CREATE_CAPTURE_INTERNAL_REPRESENTATIONS_BIT.
     */
    uint64_t *qpu_insts;
    uint32_t qpu_insts_size;
@@ -1644,6 +1749,8 @@ struct v3dv_pipeline_stage {
    uint32_t program_id;
 
    VkPipelineCreationFeedback feedback;
+
+   struct vk_pipeline_robustness_state robustness;
 };
 
 /* We are using the descriptor pool entry for two things:
@@ -1803,9 +1910,36 @@ struct v3dv_pipeline_layout {
    uint32_t dynamic_offset_count;
    uint32_t push_constant_size;
 
+   /* Pipeline layouts can be destroyed after creating pipelines since
+    * maintenance4.
+    */
+   uint32_t ref_cnt;
+
    unsigned char sha1[20];
 };
 
+void
+v3dv_pipeline_layout_destroy(struct v3dv_device *device,
+                             struct v3dv_pipeline_layout *layout,
+                             const VkAllocationCallbacks *alloc);
+
+static inline void
+v3dv_pipeline_layout_ref(struct v3dv_pipeline_layout *layout)
+{
+   assert(layout && layout->ref_cnt >= 1);
+   p_atomic_inc(&layout->ref_cnt);
+}
+
+static inline void
+v3dv_pipeline_layout_unref(struct v3dv_device *device,
+                           struct v3dv_pipeline_layout *layout,
+                           const VkAllocationCallbacks *alloc)
+{
+   assert(layout && layout->ref_cnt >= 1);
+   if (p_atomic_dec_zero(&layout->ref_cnt))
+      v3dv_pipeline_layout_destroy(device, layout, alloc);
+}
+
 /*
  * We are using descriptor maps for ubo/ssbo and texture/samplers, so we need
  * it to be big enough to include the max value for all of them.
@@ -1819,7 +1953,7 @@ struct v3dv_pipeline_layout {
 
 
 struct v3dv_descriptor_map {
-   /* TODO: avoid fixed size array/justify the size */
+   /* FIXME: avoid fixed size array/justify the size */
    unsigned num_desc; /* Number of descriptors  */
    int set[DESCRIPTOR_MAP_SIZE];
    int binding[DESCRIPTOR_MAP_SIZE];
@@ -1960,20 +2094,12 @@ struct v3dv_pipeline {
    struct v3dv_device *device;
 
    VkShaderStageFlags active_stages;
+   VkPipelineCreateFlags flags;
 
    struct v3dv_render_pass *pass;
    struct v3dv_subpass *subpass;
 
-   /* Note: We can't use just a MESA_SHADER_STAGES array because we also need
-    * to track binning shaders. Note these will be freed once the pipeline
-    * has been compiled.
-    */
-   struct v3dv_pipeline_stage *vs;
-   struct v3dv_pipeline_stage *vs_bin;
-   struct v3dv_pipeline_stage *gs;
-   struct v3dv_pipeline_stage *gs_bin;
-   struct v3dv_pipeline_stage *fs;
-   struct v3dv_pipeline_stage *cs;
+   struct v3dv_pipeline_stage *stages[BROADCOM_SHADER_STAGES];
 
    /* Flags for whether optional pipeline stages are present, for convenience */
    bool has_gs;
@@ -2006,6 +2132,7 @@ struct v3dv_pipeline {
    uint32_t sample_mask;
 
    bool primitive_restart;
+   bool negative_one_to_one;
 
    /* Accessed by binding. So vb[binding]->stride is the stride of the vertex
     * array with such binding
@@ -2072,7 +2199,6 @@ struct v3dv_pipeline {
 
    struct {
       void *mem_ctx;
-      bool has_data;
       struct util_dynarray data; /* Array of v3dv_pipeline_executable_data */
    } executables;
 
