@@ -28,7 +28,7 @@
 /* stdbool.h is necessary because this file is included in both C and C++ code.
  */
 #include <stdbool.h>
-
+#include "util/macros.h"
 #include "program/prog_parameter.h"  /* For union gl_constant_value. */
 
 /**
@@ -88,8 +88,17 @@ struct gl_opaque_uniform_index {
    bool active;
 };
 
+struct gl_resource_name
+{
+   char *string;
+   int length;              /* strlen(string) or 0 */
+   int last_square_bracket; /* (strrchr(name, '[') - name) or -1 */
+   bool suffix_is_zero_square_bracketed; /* suffix is [0] */
+};
+
 struct gl_uniform_storage {
-   char *name;
+   struct gl_resource_name name;
+
    /** Type of this uniform data stored.
     *
     * In the case of an array, it's the type of a single array element.
@@ -106,6 +115,11 @@ struct gl_uniform_storage {
 
    struct gl_opaque_uniform_index opaque[MESA_SHADER_STAGES];
 
+   /**
+    * Mask of shader stages (1 << MESA_SHADER_xxx) where this uniform is used.
+    */
+   unsigned active_shader_mask;
+
    /**
     * Storage used by the driver for the uniform
     */
@@ -201,8 +215,17 @@ struct gl_uniform_storage {
     * top-level shader storage block member. (GL_TOP_LEVEL_ARRAY_STRIDE).
     */
    unsigned top_level_array_stride;
+
+   /**
+    * Whether this uniform variable has the bindless_sampler or bindless_image
+    * layout qualifier as specified by ARB_bindless_texture.
+    */
+   bool is_bindless;
 };
 
+void
+resource_name_updated(struct gl_resource_name *name);
+
 #ifdef __cplusplus
 }
 #endif
